import troposphere.elasticloadbalancing as elb
import troposphere.elasticloadbalancingv2 as elbv2
import troposphere.ec2 as ec2

from .common import *
from .shared import (Parameter, get_endvalue, get_expvalue, get_subvalue,
                     auto_get_props, get_condition, add_obj)
from .route53 import R53_RecordSetEC2LoadBalancer, R53_RecordSetECSLoadBalancer
from .securitygroup import SecurityGroupIngressInstanceELBPorts
from .lambdas import LambdaPermissionLoadBalancing


# Temporary fix for https://github.com/cloudtools/troposphere/issues/1474
elbv2.one_of = my_one_of


# S - V2 LOAD BALANCING #
class ELBV2ListernerRuleECS(elbv2.ListenerRule):
    def __init__(self, title, key, index, mapname, scheme, **kwargs):
        super().__init__(title, **kwargs)

        auto_get_props(self, f'ListenerRules{index}')
        if 'Conditions' not in key:
            self.Conditions = []
            if 'HostHeader' in key:
                self.Conditions.append(elbv2.Condition(
                    Field='host-header',
                    Values=[get_endvalue(f'{mapname}HostHeader', issub=True)]
                ))
            if 'PathPattern' in key:
                self.Conditions.append(elbv2.Condition(
                    Field='path-pattern',
                    Values=[
                        Sub('${Value}', **{'Value': get_endvalue(
                            f'{mapname}PathPattern')})
                    ]
                ))
            if 'RequestMethod' in key:
                self.Conditions.append(elbv2.Condition(
                    Field='http-request-method',
                    HttpRequestMethodConfig=elbv2.HttpRequestMethodConfig(
                        Values=[get_endvalue(
                            f'{mapname}RequestMethod', issub=True)])
                ))
        if 'Actions' not in key:
            tg_prefix = key.get('TargetGroupPrefix', '')
            self.Actions = [elbv2.Action(
                Type='forward',
                TargetGroupArn=Ref(f'TargetGroup{tg_prefix}{scheme}')
            )]
        self.Priority = get_endvalue(f'{mapname}Priority')


# E - V2 LOAD BALANCING #


def LB_ListenersEC2():
    # Resources
    Listeners = []
    for n, v in cfg.Listeners.items():
        mapname = f'Listeners{n}'  # Ex ListenersPort5601

        if cfg.LoadBalancerClassic:
            Listener = elb.Listener(mapname)
            auto_get_props(Listener)
            auto_get_props(Listener, 'ListenerClassic')
            Listeners.append(Listener)

        # resources
        r_SGIInstance = SecurityGroupIngressInstanceELBPorts(
            f'SecurityGroupIngress{mapname}', listener=mapname)

        add_obj(r_SGIInstance)

        # outputs
        Listener_Output = Output(mapname)
        Listener_Output.Value = Sub(
            'Protocol=${Protocol},Access=${Access}',
            **{
                'Protocol': get_endvalue(
                    f'{mapname}Protocol'),
                'Access': get_endvalue(
                    f'{mapname}Access')
            }
        )
        add_obj(Listener_Output)

    return Listeners


def LB_ListenerRulesExternalInternal(index, key, mapname, scheme):
    # Skip if Rule is only for External/Internal
    only_scheme = key.get('Scheme')
    if only_scheme and only_scheme != scheme:
        return

    # resources
    R_RuleHttp = ELBV2ListernerRuleECS(
        f'ListenerHttp{scheme}Rules{index}',
        key=key, index=index, mapname=mapname, scheme=scheme)
    R_RuleHttp.ListenerArn = get_expvalue(
        f'ListenerHttpDefault{scheme}', 'LoadBalancerApplicationStack')
    R_RuleHttp.Condition = 'ListenerLoadBalancerHttpPort'

    R_RuleHttps = ELBV2ListernerRuleECS(
        f'ListenerHttps{scheme}Rules{index}',
        key=key, index=index, mapname=mapname, scheme=scheme)
    R_RuleHttps.ListenerArn = get_expvalue(
        f'ListenerHttpsDefault{scheme}', 'LoadBalancerApplicationStack')
    R_RuleHttps.Condition = 'ListenerLoadBalancerHttpsPort'

    # Create ListenerRule only in stack's specific new Listener
    ListenerHttpPort = cfg.ListenerLoadBalancerHttpPort
    ListenerHttpsPort = cfg.ListenerLoadBalancerHttpsPort

    Protocol = key['Protocol'] if 'Protocol' in key else 'auto'

    RuleHttpAdd = None
    RuleHttpsAdd = None

    if 'NoDefault' in key:
        R_RuleHttp.ListenerArn = Ref(f'ListenerHttp{scheme}')
        R_RuleHttps.ListenerArn = Ref(f'ListenerHttps{scheme}')
        if ListenerHttpPort != 80:
            RuleHttpAdd = True
        if ListenerHttpsPort != 443:
            RuleHttpsAdd = True
    else:
        # by default create http rules on Internal LB
        # and https rules on External one
        if scheme == 'Internal':
            RuleHttpAdd = True
        if scheme == 'External':
            RuleHttpsAdd = True
            # on External can be forced or overriden by key http/https/any
            if Protocol == 'http':
                RuleHttpAdd = True
                RuleHttpsAdd = None
            if Protocol == 'https':
                RuleHttpAdd = None
                RuleHttpsAdd = True
            if Protocol == 'any':
                RuleHttpAdd = True
                RuleHttpsAdd = True

    if RuleHttpAdd:
        add_obj(R_RuleHttp)
    if RuleHttpsAdd:
        add_obj(R_RuleHttps)


def LB_ListenerRules():
    for n, v in cfg.ListenerRules.items():
        mapname = f'ListenerRules{n}'  # Ex. ListenerRules1

        # parameters
        p_Priority = Parameter(f'{mapname}Priority')
        p_Priority.Description = (
            'Listener Rule Priority, lesser value = high priority - '
            'empty for default based on env/role')

        add_obj(p_Priority)

        ListenerRule_Out_String = ['Priority=${Priority}']
        ListenerRule_Out_Map = {'Priority': get_endvalue(
            f'{mapname}Priority')}

        if 'HostHeader' in v:
            p_HostHeader = Parameter(f'{mapname}HostHeader')
            p_HostHeader.Description = (
                'Listener Rule HostHeader Condition - '
                'empty for default based on env/role')

            add_obj(p_HostHeader)

            # outputs
            ListenerRule_Out_String.append('HostHeader=${HostHeader}')
            ListenerRule_Out_Map.update({
                'HostHeader': get_endvalue(
                    f'{mapname}HostHeader', issub=True)
            })

        if 'PathPattern' in v:
            p_PathPattern = Parameter(f'{mapname}PathPattern')
            p_PathPattern.Description = (
                'Listener Rule PathPattern Condition - '
                'empty for default based on env/role')

            add_obj(p_PathPattern)

            # outputs
            ListenerRule_Out_String.append('PathPattern=${PathPattern}')
            ListenerRule_Out_Map.update({
                'PathPattern': get_endvalue(
                    f'{mapname}PathPattern', issub=True)
            })

        # resources
        if cfg.LoadBalancerApplicationExternal:
            LB_ListenerRulesExternalInternal(
                index=str(n), key=v, mapname=mapname, scheme='External')

        if cfg.LoadBalancerApplicationInternal:
            LB_ListenerRulesExternalInternal(
                index=str(n), key=v, mapname=mapname, scheme='Internal')

        # outputs
        o_ListenerRule = Output(mapname)
        o_ListenerRule.Value = Sub(
            ','.join(ListenerRule_Out_String), **ListenerRule_Out_Map)

        add_obj(o_ListenerRule)


def LB_ListenersV2EC2():
    LB_ListenersEC2()
    for n in ['External', 'Internal']:
        # resources
        if n not in cfg.LoadBalancerApplication:
            continue
        r_Http = elbv2.Listener(f'ListenerHttp{n}')
        auto_get_props(r_Http, mapname=f'ListenerV2EC2Http{n}')
        add_obj(r_Http)

        if n == 'External':
            r_Https = elbv2.Listener(f'ListenerHttps{n}')
            auto_get_props(r_Https, mapname=f'ListenerV2EC2Https{n}')
            add_obj(r_Https)

    LB_TargetGroupsEC2()


def LB_ListenersV2ECS():
    for n in ['External', 'Internal']:
        # resources
        if n not in cfg.LoadBalancerApplication:
            continue
        if cfg.ListenerLoadBalancerHttpPort not in ['none', 80]:
            # Enable the relative LB SecurityGroupIngress
            cfg.SecurityGroupIngress[
                f'LoadBalancerApplicationHttp{n}']['IBOX_ENABLED'] = True
            r_Http = elbv2.Listener(f'ListenerHttp{n}')
            auto_get_props(r_Http, mapname=f'ListenerV2ECSHttp{n}')
            add_obj(r_Http)

        if (cfg.ListenerLoadBalancerHttpsPort not in ['none', 443]
                and n == 'External'):
            # Enable the relative LB SecurityGroupIngress
            cfg.SecurityGroupIngress[
                f'LoadBalancerApplicationHttps{n}']['IBOX_ENABLED'] = True
            r_Https = elbv2.Listener(f'ListenerHttps{n}')
            auto_get_props(r_Https, mapname=f'ListenerV2ECSHttps{n}')
            add_obj(r_Https)

    # Resources
    LB_TargetGroupsECS()
    LB_ListenerRules()


def LB_TargetGroupsEC2():
    for n in cfg.ElasticLoadBalancingV2TargetGroupEC2:
        # resources
        if n not in cfg.LoadBalancerApplication:
            continue
        r_TG = elbv2.TargetGroup(f'TargetGroup{n}')
        auto_get_props(r_TG,
                       mapname=f'ElasticLoadBalancingV2TargetGroupEC2{n}')
        add_obj(r_TG)

        cfg.Alarm[f'TargetEC2{n}5XX']['IBOX_ENABLED'] = True


def LB_TargetGroupsECS():
    for n in cfg.ElasticLoadBalancingV2TargetGroupECS:
        # resources
        if (n in ['External', 'Internal']
                and n not in cfg.LoadBalancerApplication):
            continue
        r_TG = elbv2.TargetGroup(f'TargetGroup{n}')
        auto_get_props(r_TG,
                       mapname=f'ElasticLoadBalancingV2TargetGroupECS{n}')
        add_obj(r_TG)

        try:
            cfg.Alarm[f'Target{n}5XX']['IBOX_ENABLED'] = True
        except Exception:
            pass


def LB_TargetGroupsALB():
    lambda_name = 'ServiceUnavailable'
    lambda_arn = get_expvalue(f'Lambda{lambda_name}Arn')
    perm_name = f'LambdaPermission{lambda_name}LoadBalancerApplication'

    for n in cfg.ElasticLoadBalancingV2TargetGroupALB:
        # resources
        if n not in cfg.LoadBalancerApplication:
            continue
        r_TG = elbv2.TargetGroup(f'TargetGroupServiceUnavailable{n}')
        auto_get_props(r_TG,
                       mapname=f'ElasticLoadBalancingV2TargetGroupALB{n}')
        r_TG.DependsOn = f'{perm_name}{n}'
        r_TG.Condition = f'LoadBalancerApplication{n}'

        r_LambdaPermission = LambdaPermissionLoadBalancing(
            f'{perm_name}{n}', name=lambda_arn)
        r_LambdaPermission.Condition = f'LoadBalancerApplication{n}'

        o_TargetGroup = Output(f'TargetGroupServiceUnavailable{n}')
        o_TargetGroup.Condition = f'LoadBalancerApplication{n}'
        o_TargetGroup.Value = Ref(f'TargetGroupServiceUnavailable{n}')

        add_obj([
            r_TG,
            r_LambdaPermission,
            o_TargetGroup])


def LB_ElasticLoadBalancingClassicEC2():
    Listeners = LB_ListenersEC2()
    for n, v in cfg.ElasticLoadBalancingLoadBalancer.items():
        # resources
        if n not in cfg.LoadBalancerClassic:
            continue
        r_LB = elb.LoadBalancer(f'LoadBalancerClassic{n}')
        auto_get_props(r_LB, mapname=f'ElasticLoadBalancingLoadBalancer{n}')
        r_LB.Listeners = Listeners

        add_obj(r_LB)
        cfg.Alarm[f'Backend{n}5XX']['IBOX_ENABLED'] = True


def LB_ElasticLoadBalancingApplicationEC2():
    for n, v in cfg.ElasticLoadBalancingV2LoadBalancer.items():
        # resources
        if n not in cfg.LoadBalancerApplication:
            continue
        r_LB = elbv2.LoadBalancer(f'LoadBalancerApplication{n}')
        auto_get_props(r_LB, mapname=f'ElasticLoadBalancingV2LoadBalancer{n}')
        add_obj(r_LB)

    LB_ListenersV2EC2()


def LB_ElasticLoadBalancingEC2(key):
    # Resources
    R53_RecordSetEC2LoadBalancer()

    if cfg.LoadBalancerClassic:
        LB_ElasticLoadBalancingClassicEC2()

    if cfg.LoadBalancerApplication:
        LB_ElasticLoadBalancingApplicationEC2()


def LB_ElasticLoadBalancingALB(key):
    for n, v in cfg.ElasticLoadBalancingV2LoadBalancerALB.items():
        # resources
        if n not in cfg.LoadBalancerApplication:
            continue
        r_LB = elbv2.LoadBalancer(f'LoadBalancerApplication{n}')
        auto_get_props(r_LB,
                       mapname=f'ElasticLoadBalancingV2LoadBalancerALB{n}')

        r_ListenerHttp = elbv2.Listener(f'ListenerHttpDefault{n}')
        auto_get_props(r_ListenerHttp, mapname=f'ListenerV2ALBHttp{n}')

        if n == 'External':
            # Https enabled only for External ELB
            r_ListenerHttps = elbv2.Listener(f'ListenerHttpsDefault{n}')
            auto_get_props(r_ListenerHttps, mapname=f'ListenerV2ALBHttps{n}')
            add_obj(r_ListenerHttps)

        add_obj([
            r_LB,
            r_ListenerHttp])

    # Resources
    # Create TargetGroups pointing to LambdaServiceUnavailable
    try:
        cfg.ServiceUnavailable
    except Exception:
        pass
    else:
        LB_TargetGroupsALB()


def LB_ElasticLoadBalancingECS(key):
    # Resources
    LB_ListenersV2ECS()
    R53_RecordSetECSLoadBalancer()



import os
import sys
from functools import reduce

cwd = os.getcwd()
sys.path.append(os.path.join(cwd, 'lib'))

no_override = False

Parameters = {}
Conditions = {}
Mappings = {}
Resources = {}
Outputs = {}

PATH_INT = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'cfg')
PATH_INT = os.path.normpath(PATH_INT)

PATH_EXT = os.path.join(cwd, 'cfg')
PATH_EXT = os.path.normpath(PATH_EXT)

STACK_TYPES = [
    'agw',
    'alb',
    'cch',
    'clf',
    'ec2',
    'ecr',
    'ecs',
    'rds',
    'res',
    'tsk',
]

MAX_SECURITY_GROUPS = 4
# SECURITY_GROUPS_DEFAULT equals list of empty values
# (Ex. for "MAX_SECURITY_GROUPS = 3" we have "SECURITY_GROUPS_DEFAULT = ',,'")
SECURITY_GROUPS_DEFAULT = reduce(
    lambda a, i: f',{a}', list(range(MAX_SECURITY_GROUPS - 1)), '')

ENV_BASE = ['dev', 'stg', 'prd']

DEFAULT_REGIONS = [
    'eu-west-1',
    'us-east-1',
    'eu-central-1',
]

AZones = {
    'MAX': 6,
    'default': 3,
    'us-east-1': 6,
    'us-west-1': 2,
    'us-west-2': 4,
    'ca-central-1': 2,
    'sa-east-1': 2,
    'cn-north-1': 2,
    'ap-northeast-3': 1,
}

AZoneNames = ['A', 'B', 'C', 'D', 'E', 'F']

PARAMETERS_SKIP_OVERRIDE_CONDITION = (
    'Env',
    'UpdateMode',
    'RecordSetExternal',
    'DoNotSignal',
    'EfsMounts',
    'LaunchTemplateDataImageIdLatest',
    'VPCCidrBlock',
    'VPCName',
)

EVAL_FUNCTIONS_IN_CFG = (
    'cfg.',
    'get_expvalue(',
    'Sub(',
    'Ref(',
    'get_subvalue(',
    'get_endvalue(',
    'get_resvalue(',
    'get_condition(',
    'GetAtt(',
    'Split(',
    'Export(',
    'ImportValue(',
    'Join(',
    'Base64(',
    'If(',
    'Equals(',
    'Not(',
    'GetAZs(',
    'dict(',
    'eval(',
    'Tags(',
    'str(',
    'list(',
    'SG_SecurityGroups',
)

CLF_PATH_PATTERN_REPLACEMENT = {
    '/': 'SLASH',
    '*': 'STAR',
    '-': 'HYPH',
    '?': 'QUEST',
    '.': 'DOT',
    '_': 'USCORE',
}

BASE_CFGS = {
    'Bucket': {
        'Create': 'no',
        'AccountsRead': {
            'dev': 'none',
            'stg': 'none',
            'prd': 'none',
        },
        'AccountsWrite': {
            'dev': 'none',
            'stg': 'none',
            'prd': 'none',
        },
        'Cors': 'no',
        'CloudFrontOriginAccessIdentityExtra': {
            'Dev': 'none',
            'Stg': 'none',
            'Prd': 'none',
        },
        'PolicyStatementReplica': {
            'Resource': {},
            'Principal': 'none',
        },
        'Replication': {
            'Enabled': 'no',
            'ConfigurationRules': {},
        },
        'Versioning': 'Disabled',
    },
    'CloudFrontCacheBehaviors': {
        'Compress': True,
        'ForwardedValues': {
            'QueryString': True,
            'Headers': []
        },
        'ViewerProtocolPolicy': 'redirect-to-https'
    },
    'CloudFrontOrigins': 'IBOX_BASE',
    'DBInstance': 'IBOX_BASE',
    'ScheduledAction': 'IBOX_BASE',
    'ContainerDefinitions': 'IBOX_BASE',
    'AllowedIp': 'IBOX_BASE',
    'ElasticLoadBalancingV2TargetGroupECS': 'IBOX_BASE',
    'Certificate': 'IBOX_BASE',
    'CloudFrontCachePolicy': 'IBOX_BASE',
    'CloudFrontOriginRequestPolicy': 'IBOX_BASE',
}

INSTANCE_LIST = [
    'default',
    't2.nano', 't2.micro', 't2.small', 't2.medium', 't2.large',
    't2.xlarge', 't2.2xlarge',
    'm4.large', 'm4.xlarge', 'm4.2xlarge', 'm4.4xlarge',
    'c4.large', 'c4.xlarge', 'c4.2xlarge', 'c4.4xlarge',
    'r4.large', 'r4.xlarge', 'r4.2xlarge', 'r4.4xlarge',
    'g2.2xlarge', 'g3s.xlarge',
    't3.nano', 't3.micro', 't3.small', 't3.medium', 't3.large',
    't3.xlarge', 't3.2xlarge',
    'm5.large', 'm5.xlarge', 'm5.2xlarge', 'm5.4xlarge', 'm5.8xlarge',
    'm5.12xlarge', 'm5.16xlarge', 'm5.24xlarge',
    'c5.large', 'c5.xlarge', 'c5.2xlarge', 'c5.4xlarge', 'c5.9xlarge',
    'c5.12xlarge', 'c5.18xlarge', 'c5.24xlarge',
    'r5.large', 'r5.xlarge', 'r5.2xlarge', 'r5.4xlarge', 'r5.8xlarge',
    'r5.12xlarge', 'r5.16xlarge', 'r5.24xlarge',
    'a1.medium', 'a1.large', 'a1.xlarge', 'a1.2xlarge', 'a1.4xlarge',
    'm5a.large', 'm5a.xlarge', 'm5a.2xlarge', 'm5a.4xlarge', 'm5a.8xlarge',
    'm5a.12xlarge', 'm5a.16xlarge', 'm5a.24xlarge',
    'r5a.large', 'r5a.xlarge', 'r5a.2xlarge', 'r5a.4xlarge', 'r5a.8xlarge',
    'r5a.12xlarge', 'r5a.16xlarge', 'r5a.24xlarge',
    'c5a.large', 'c5a.xlarge', 'c5a.2xlarge', 'c5a.4xlarge', 'c5a.8xlarge',
    'c5a.12xlarge', 'c5a.16xlarge', 'c5a.24xlarge',
    't3a.nano', 't3a.micro', 't3a.small', 't3a.medium', 't3a.large',
    't3a.xlarge', 't3a.2xlarge',
    't4g.nano', 't4g.micro', 't4g.small', 't4g.medium', 't4g.large',
    't4g.xlarge', 't4g.2xlarge',
    'c6g.medium', 'c6g.large', 'c6g.xlarge', 'c6g.2xlarge', 'c6g.4xlarge',
    'c6g.8xlarge', 'c6g.12xlarge', 'c6g.16xlarge',
    'm6g.medium', 'm6g.large', 'm6g.xlarge', 'm6g.2xlarge', 'm6g.4xlarge',
    'm6g.8xlarge', 'm6g.12xlarge', 'm6g.16xlarge',
]

# override previous cfg with an External one
try:
    with open(os.path.join(cwd, 'lib', 'cfgExt.py')) as f:
        exec(f.read())
except FileNotFoundError:
    pass

# Order is VERY important do not CHANGE it!
CFG_TO_FUNC = {
    'MappingClass': {
        'module': 'mappings',
        'func': 'Mappings'},
    'Parameter': {
        'module': 'cloudformation',
        'func': 'CFM_Parameters'},
    'Condition': {
        'module': 'cloudformation',
        'func': 'CFM_Conditions'},
    'Mapping': {
        'module': 'cloudformation',
        'func': 'CFM_Mappings'},
    # SecurityGroups need to stay here because it populate a cfg value
    # [cfg.SecurityGroupsImport] used later by other keys/modules
    'SecurityGroups': {
        'module': 'securitygroup',
        'func': 'SG_SecurityGroups'},
    'AutoScalingGroup': {
        'module': 'autoscaling',
        'func': 'AS_Autoscaling'},
    'ScheduledAction': {
        'module': 'joker',
        'func': ('autoscaling', 'ScheduledAction')},
    'ScalableTarget': {
        'module': 'autoscaling',
        'func': 'AS_ScalableTarget'},
    'AutoScalingScalingPolicy': {
        'module': 'autoscaling',
        'func': 'AS_ScalingPolicies'},
    'ApplicationAutoScalingScalingPolicy': {
        'module': 'autoscaling',
        'func': 'AS_ScalingPolicies'},
    'Bucket': {
        'module': 's3',
        'func': 'S3_Buckets'},
    'Certificate': {
        'module': 'joker',
        'func': ('certificatemanager', 'Certificate')},
    'CodeDeployApp': {
        'module': 'codedeploy',
        'func': 'CD_Applications'},
    'Repository': {
        'module': 'ecr',
        'func': 'ECR_Repositories'},
    'ECSCluster': {
        'module': 'joker',
        'func': ('ecs', 'Cluster')},
    'ECSCapacityProvider': {
        'module': 'joker',
        'func': ('ecs', 'CapacityProvider')},
    'ECSClusterCapacityProviderAssociations': {
        'module': 'joker',
        'func': ('ecs', 'ClusterCapacityProviderAssociations')},
    'Service': {
        'module': 'ecs',
        'func': 'ECS_Service'},
    'ApiGatewayAccount': {
        'module': 'apigateway',
        'func': 'AGW_Account'},
    'ApiGatewayDomainName': {
        'module': 'apigateway',
        'func': 'AGW_DomainName'},
    'ApiGatewayRestApi': {
        'module': 'apigateway',
        'func': 'AGW_RestApi'},
    'ApiGatewayBasePathMapping': {
        'module': 'joker',
        'func': ('apigateway', 'BasePathMapping')},
    'ApiGatewayStage': {
        'module': 'apigateway',
        'func': 'AGW_Stages'},
    'ApiGatewayUsagePlan': {
        'module': 'apigateway',
        'func': 'AGW_UsagePlans'},
    'ApiGatewayApiKey': {
        'module': 'apigateway',
        'func': 'AGW_ApiKeys'},
    'LogGroupName': {
        'module': 'logs',
        'func': 'LGS_LogGroup'},
    'TaskDefinition': {
        'module': 'ecs',
        'func': 'ECS_TaskDefinition'},
    'EFSFileSystem': {
        'module': 'efs',
        'func': 'EFS_FileStorage'},
    'EFSAccessPoint': {
        'module': 'joker',
        'func': ('efs', 'AccessPoint')},
    'CacheSubnetGroup': {
        'module': 'elasticache',
        'func': 'CCH_SubnetGroups'},
    'CacheCluster': {
        'module': 'elasticache',
        'func': 'CCH_Cache'},
    'EventsRule': {
        'module': 'events',
        'func': 'EVE_EventRules'},
    'Role': {
        'module': 'iam',
        'func': 'IAM_Roles'},
    'IAMPolicy': {
        'module': 'iam',
        'func': 'IAM_Policies'},
    'IAMUser': {
        'module': 'iam',
        'func': 'IAM_Users'},
    'IAMGroup': {
        'module': 'iam',
        'func': 'IAM_Groups'},
    'IAMUserToGroupAddition': {
        'module': 'iam',
        'func': 'IAM_UserToGroupAdditions'},
    'KMSKey': {
        'module': 'kms',
        'func': 'KMS_Keys'},
    'Lambda': {
        'module': 'lambdas',
        'func': 'LBD_Lambdas'},
    'LambdaLayerVersion': {
        'module': 'lambdas',
        'func': 'LBD_LayerVersions'},
    'LambdaPermission': {
        'module': 'joker',
        'func': ('awslambda', 'Permission')},
    'LambdaEventSourceMapping': {
        'module': 'joker',
        'func': ('awslambda', 'EventSourceMapping')},
    'LambdaEventInvokeConfig': {
        'module': 'joker',
        'func': ('awslambda', 'EventInvokeConfig')},
    'LoadBalancerApplication': {
        'module': 'loadbalancing',
        'func': 'LB_ElasticLoadBalancing'},
    'LoadBalancerClassic': {
        'module': 'loadbalancing',
        'func': 'LB_ElasticLoadBalancing'},
    'Alarm': {
        'module': 'joker',
        'func': ('cloudwatch', 'Alarm')},
    'CloudFrontDistribution': {
        'module': 'cloudfront',
        'func': 'CF_CloudFront'},
    'CloudFrontCachePolicy': {
        'module': 'joker',
        'func': ('cloudfront', 'CachePolicy')},
    'CloudFrontOriginRequestPolicy': {
        'module': 'joker',
        'func': ('cloudfront', 'OriginRequestPolicy')},
    'SNSSubscription': {
        'module': 'sns',
        'func': 'SNS_Subscriptions'},
    'SNSTopic': {
        'module': 'sns',
        'func': 'SNS_Topics'},
    'SQSQueue': {
        'module': 'joker',
        'func': ('sqs', 'Queue')},
    'ASGLifecycleHook': {
        'module': 'joker',
        'func': ('autoscaling', 'LifecycleHook')},
    'DBInstance': {
        'module': 'rds',
        'func': 'RDS_DB'},
    'DBSubnetGroup': {
        'module': 'rds',
        'func': 'RDS_SubnetGroups'},
    'HostedZone': {
        'module': 'route53',
        'func': 'R53_HostedZones'},
    'R53RecordSet': {
        'module': 'route53',
        'func': 'R53_RecordSets'},
    'WafByteMatchSet': {
        'module': 'waf',
        'func': ['WAF_GlobalByteMatchSets', 'WAF_RegionalByteMatchSets']},
    'WafIPSet': {
        'module': 'waf',
        'func': ['WAF_GlobalIPSets', 'WAF_RegionalIPSets']},
    'WafRule': {
        'module': 'waf',
        'func': ['WAF_GlobalRules', 'WAF_RegionalRules']},
    'WafWebAcl': {
        'module': 'waf',
        'func': ['WAF_GlobalWebAcls', 'WAF_RegionalWebAcls']},
    'ServiceDiscoveryPublicDnsNamespace': {
        'module': 'joker',
        'func': ('servicediscovery', 'PublicDnsNamespace')},
    'ServiceDiscoveryService': {
        'module': 'joker',
        'func': ('servicediscovery', 'Service')},
    'VPCEndpoint': {
        'module': 'vpc',
        'func': 'VPC_Endpoint'},
    'SecurityGroup': {
        'module': 'securitygroup',
        'func': 'SG_SecurityGroup'},
    'SecurityGroupIngress': {
        'module': 'securitygroup',
        'func': 'SG_SecurityGroupIngresses'},
    'VPC': {
        'module': 'vpc',
        'func': 'VPC_VPC'},
    'CCRLightHouse': {
        'module': 'cloudformation',
        'func': 'CFM_CustomResourceLightHouse'},
    'CCRFargateSpot': {
        'module': 'cloudformation',
        'func': 'CFM_CustomResourceFargateSpot'},
    # ReplicateRegions need to stay here
    'CCRReplicateRegions': {
        'module': 'cloudformation',
        'func': 'CFM_CustomResourceReplicator'},
    # Output need to be last line
    'Output': {
        'module': 'cloudformation',
        'func': 'CFM_Outputs'},
}



import troposphere.sqs as sqs

from .common import *
from .shared import (Parameter, get_endvalue, get_expvalue, get_subvalue,
                     auto_get_props, get_condition, add_obj)


class SQSQueuePolicy(sqs.QueuePolicy):
    def __init__(self, title, key, **kwargs):
        super().__init__(title, **kwargs)
        self.Queues = [
            Sub(
                'https://sqs.${AWS::Region}.amazonaws.com/'
                '${AWS::AccountId}/${QueueId}',
                **{'QueueId': Select('5', Split(':', eval(key['Endpoint'])))}
            )
        ]
        self.PolicyDocument = {
            'Version': '2012-10-17',
            'Statement': [
                {
                    'Action': [
                        'SQS:SendMessage'
                    ],
                    'Condition': {
                        'ArnEquals': {
                            'aws:SourceArn': eval(key['TopicArn'])
                        }
                    },
                    'Effect': 'Allow',
                    'Principal': {
                        'AWS': '*'
                    },
                    'Resource': '*'
                }
            ]
        }
